@@ -18,7 +18,7 @@ from math import pi
 import sys
 
 ################
-VERSION="1.0.1"
+VERSION="1.0.2"
 ################
 
 def insertCopyNumber(outfilename, filename):
@@ -60,6 +60,7 @@ def getArgsFromCommandLine():
     parser.add_argument('-vertextol', help=formatting('Default 0.25 degrees. This sets the vertex tolerance within which two vertices of a triangle must match in angle to another triangle in order to be considered a potential pattern-match between the image source coordinates and catalogue source coordinates. Not much testing has been done on this option and 0.25 has always been used. A smaller tolerance will be less forgiving to distortion but would offer faster searching and fewer false-positives, whereas a larger tolerance will be more forgiving to distortion at the expense of slower searching and a higher rate of false-positives.'))
     parser.add_argument('-nmatchpoints', help=formatting('This specifies the number of points which must match between the point source list and catalogue list to be considered a solution. Default is 6 points. Minimum is 3 but this is very likely to produce false-positive solutions. 4 or 5 can be used where there are simply very few points in the source image. 6 or higher should guarantee no false positives.'))
     parser.add_argument('-nmatchpercent', help=formatting('Same as above but as a percentage of point sources available. Default is 25%%. If both nmatchpoints and nmatchpercent are supplied, the smaller threshold will be used, found by comparing nmatchpoints to nmatchpercent*npts.'))
+    parser.add_argument('-wcsdiagnostics', action='store_true', help=formatting(''))
     parser.add_argument('--save', metavar='ID', help=formatting('Save the options settings as a file to disk to recall later. User supplies an ID, for example: "--save UVIT". Useful if frequently processing different telescopic fields, allowing quick input of WCS solver settings. Saves all settings EXCEPT for filename, ra, dec, pmepoch, so that different images but taken from the same telescope can be processed at their unique individual characteristics.'))
     parser.add_argument('--load', metavar='ID', help=formatting('Loads options from the json file whose filename contains the ID specified. Any supplied options will be overwritten by the ones in the json file.'))
     parser.add_argument('--version', '-V', action='version', version='fastrometry {}'.format(VERSION), help=formatting('Shows the current version of fastrometry.'))
@@ -70,7 +71,7 @@ def getArgsFromCommandLine():
 
     return input_args
 
-def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotation=None, rotationbnds=None, buffer=None, shape=None, catalogue=None, pmepoch=None, filter=None, npts=None, nrefinepts=None, pixsat=None, kernelrad=None, sourcesep=None, vertextol=None, nmatchpoints=None, nmatchpercent=None, save=None, load=None, verbosity=None, debug=None):
+def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotation=None, rotationbnds=None, buffer=None, shape=None, catalogue=None, pmepoch=None, filter=None, npts=None, nrefinepts=None, pixsat=None, kernelrad=None, sourcesep=None, vertextol=None, nmatchpoints=None, nmatchpercent=None, wcsdiagnostics=None, save=None, load=None, verbosity=None, debug=None):
 
     user_dir = str(Path.cwd())
 
@@ -86,8 +87,11 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
         except:
             sys.exit('ERROR: verbosity must be an integer.')        
 
-    if debug is True:
-        verbosity = 2
+    if debug is None:
+        debug = False
+    else:
+        if debug is True:
+            verbosity = 2
         
     if verbosity >= 1:
         print("\nParsing arguments...")
@@ -111,62 +115,62 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
     if ra is None:
         sys.exit('ERROR: -ra is required.')          
     else:
-        if len(ra.split(':')) == 1:
-            try:
-                ra = float(ra)              ###decimal
-                if 0 <= ra < 360:
-                    pass
-                else:
-                    sys.exit('ERROR: ra in degree.decimal format must be between 0 and 360.')                      
-            except:                            
-                if ra in header:       ###header keywords
+        try:
+            ra = float(ra)
+            if 0 <= ra <= 360:
+                pass
+            else:
+                sys.exit('ERROR: ra in degree.decimal format must be between 0 and 360.')
+        except:
+            if len(ra.split(':')) == 1:
+                if ra in header:
                     ra = float(header['{}'.format(ra)])
                 else:
-                    sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming ra was given as a keyword).')                      
-        elif len(ra.split(':')) == 3:      ###sexagesimal
-            ra_hms = ra.split(':')
-            try:
-                hrs = int(ra_hms[0])
-                mins = int(ra_hms[1])
-                secs = float(ra_hms[2])
-                if 0 <= hrs < 24 and  0 <= mins < 60 and 0 <= secs < 60:
-                    ra = (hrs + mins/60 + secs/3600)*15
-                else:
-                    sys.exit('ERROR: ra in HH:MM:SS.S format must hrs between 0 and 24, minutes between 0 and 60, and seconds between 0 and 60)')                     
-            except:
-                sys.exit('ERROR: ra in HH:MM:SS.S format must have integer hours, integer minutes, and integer or decimal seconds.')             
-        else:
-            sys.exit('ERROR: ra must be entered either in HH:MM:SS.S format (e.g. "18:44:27.891"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "RA_PT").')
+                    sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming ra was given as a keyword).')                          
+            elif len(ra.split(':')) == 3:      ###sexagesimal
+                ra_hms = ra.split(':')
+                try:
+                    hrs = int(ra_hms[0])
+                    mins = int(ra_hms[1])
+                    secs = float(ra_hms[2])
+                    if 0 <= hrs < 24 and  0 <= mins < 60 and 0 <= secs < 60:
+                        ra = (hrs + mins/60 + secs/3600)*15
+                    else:
+                        sys.exit('ERROR: ra in HH:MM:SS.S format must hrs between 0 and 24, minutes between 0 and 60, and seconds between 0 and 60)')                     
+                except:
+                    sys.exit('ERROR: ra in HH:MM:SS.S format must have integer hours, integer minutes, and integer or decimal seconds.')             
+            else:
+                sys.exit('ERROR: ra must be entered either in HH:MM:SS.S format (e.g. "18:44:27.891"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "RA_PT").')
             
     if dec is None:
         sys.exit('ERROR: -dec is required.')         
     else:
-        if len(dec.split(':')) == 1:
-            try:
-                dec = float(dec)                 ###decimal
-                if -90 <= dec < 90:
-                    pass
-                else:
-                    sys.exit('ERROR: dec in degree.decimal format must be between -90 and 90.')                     
-            except:
-                if dec in header:        ###header keywords
+        try:
+            dec = float(dec)                
+            if -90 <= dec < 90:
+                pass
+            else:
+                sys.exit('ERROR: dec in degree.decimal format must be between -90 and 90.')                     
+        except:
+            if len(dec.split(':')) == 1:
+                if dec in header: 
                     dec = float(header['{}'.format(dec)])
                 else:
                     sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming dec was given as a keyword).')                      
-        elif len(dec.split(':')) == 3:     ###sexagesimal
-            dec_hms = dec.split(':')
-            try:
-                degs = int(dec_hms[0])
-                amins = int(dec_hms[1])
-                asecs = float(dec_hms[2])
-                if -90 <= degs < 90 and 0 <= amins < 60 and 0 <= asecs < 60:
-                    dec = degs + amins/60 + asecs/3600
-                else:
-                    sys.exit('ERROR: dec in DD:MM:SS.S" format must have degrees between -90 and 90, arcminutes between 0 and 60, and arcseconds between 0 and 60.')                      
-            except:
-                sys.exit('ERROR: dec in DD:MM:SS.S" format must have integer degrees, integer arcminutes, and integer or decimal arcseconds.')                
-        else:
-            sys.exit('ERROR: dec must be entered either in DD:MM:SS.S" format (e.g. "-49:12:09.255"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "DEC_PT").')             
+            elif len(dec.split(':')) == 3:
+                dec_hms = dec.split(':')
+                try:
+                    degs = int(dec_hms[0])
+                    amins = int(dec_hms[1])
+                    asecs = float(dec_hms[2])
+                    if -90 <= degs < 90 and 0 <= amins < 60 and 0 <= asecs < 60:
+                        dec = degs + amins/60 + asecs/3600
+                    else:
+                        sys.exit('ERROR: dec in DD:MM:SS.S" format must have degrees between -90 and 90, arcminutes between 0 and 60, and arcseconds between 0 and 60.')                      
+                except:
+                    sys.exit('ERROR: dec in DD:MM:SS.S" format must have integer degrees, integer arcminutes, and integer or decimal arcseconds.')                
+            else:
+                sys.exit('ERROR: dec must be entered either in DD:MM:SS.S" format (e.g. "-49:12:09.255"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "DEC_PT").')             
 
     if pmepoch is None:
         pmepoch = 0
@@ -404,6 +408,7 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                     'vertextol' : vertextol,
                     'nmatchpoints' : nmatchpoints,
                     'nmatchpercent' : nmatchpercent,
+                    'wcsdiagnostics' : wcsdiagnostics,
                 }
                 id = save
                 jobject = json.dumps(contents)
@@ -442,6 +447,7 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                 vertextol = jobject['vertextol']
                 nmatchpoints = jobject['nmatchpoints']
                 nmatchpercent = jobject['nmatchpercent']
+                wcsdiagnostics = jobject['wcsdiagnostics']
                 
                 filepath = user_dir+'\\'+filename
                 hdul = fits.open('{}'.format(filepath))
@@ -477,11 +483,11 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
     ###     srcindexmap:            The source index map. This is a copy of the image with every point set to a value of -1 initially. In the PSE function, circular areas (the kernel) around each found source are given
     ###                             non-zero values. This provides a way of checking if an inverse-transformed point in the WCS function "lands" on a source.
     ###     pse_metadata:           The source metadata array, which stores the coordinates, brightness, and background of every found source from the PSE (up to nrefinepts).           
-    ###     allintrmcoords_view:    An array holding the projected coordinates ("intermediate coordinates") of a set of sky coordinates downloaded from the catalogue. The success of the WCS is contingent on the geometric
+    ###     allintrmpoints_view:    An array holding the projected coordinates ("intermediate coordinates") of a set of sky coordinates downloaded from the catalogue. The success of the WCS is contingent on the geometric
     ###                             correspondence between the intermediate coordinates and the pse coordinates.
-    ###     psecoords_view:         An array holding the pixel coordinates of the npts brightest sources from the pse_metadata. The "view" suffix indicates that it is a "memory view", a type of high-efficency array
+    ###     psepoints_view:         An array holding the pixel points of the npts brightest sources from the pse_metadata. The "view" suffix indicates that it is a "memory view", a type of high-efficency array
     ###                             pointer datatype in Cython.
-    ###     intrmcoords_view:       The equivalent of the above except for holding intermediate coordinates.                 
+    ###     intrmpoints_view:       The equivalent of the above except for holding intermediate points.                 
     ###     S                       The scale parameter found by the optimizer when finding the initial solution in the WCS.
     ###     phi                     The rotation parameter found by the optimizer when finding the initial solution in the WCS.
     ###     CRPIXx                  The reference pixel location's x coordinate, also found by the optimizer in the WCS. (Once for the initial solution, and then improved
@@ -518,12 +524,13 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
     pse_metadata_inv[:,1] = img_ymax - pse_metadata[:,1]
     pse_metadata_inv[:,2] = pse_metadata[:,2]
 
-    allintrmcoords = np.zeros((nrefinepts,2))
-    meancatcoords = np.zeros(2)
+    allintrmpoints = np.zeros((nrefinepts,2))
+    catalog_points = np.zeros((nrefinepts,2))
+    mean_catcoords = np.zeros(2)
 
     if verbosity >= 1:
         print("\nGetting sources from catalogue...")
-    num_catsources = getIntermediateCoords(ra, dec, scale, img_xmax, img_ymax, shape, filter, catalogue, pmepoch, nrefinepts, allintrmcoords, meancatcoords, user_dir, debug_report, verbosity, debug)
+    num_catsources = getIntermediateCoords(ra, dec, scale, img_xmax, img_ymax, shape, filter, catalogue, pmepoch, nrefinepts, allintrmpoints, catalog_points, mean_catcoords, user_dir, debug_report, verbosity, debug)
     if verbosity >= 1:
         print("done")
 
@@ -537,11 +544,11 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
         minmatches = nmatchpercent*npts
     kerneldiam = kernelrad*2+1
 
-    headervals = np.zeros(12)
+    headervals = np.zeros(26)
 
     if verbosity >= 1:
         print("\nPerforming WCS...")
-    WCS(scale, scalebnds, rotation, rotationbnds, npts, nrefinepts, vertextol, allintrmcoords, meancatcoords, pse_metadata, pse_metadata_inv, srcindexmap_initial, srcindexmap_refine, img_xmax, img_ymax, minmatches, kerneldiam, num_psesources, num_catsources, headervals, user_dir, debug_report, filename, verbosity, debug)
+    WCS(scale, scalebnds, rotation, rotationbnds, npts, nrefinepts, vertextol, allintrmpoints, catalog_points, mean_catcoords, pse_metadata, pse_metadata_inv, srcindexmap_initial, srcindexmap_refine, img_xmax, img_ymax, minmatches, kerneldiam, num_psesources, num_catsources, headervals, wcsdiagnostics, user_dir, debug_report, filename, verbosity, debug)
     if verbosity >= 1:
         print("done")
 
@@ -559,7 +566,17 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
     header['CDELT2'] = (headervals[9], 'WCS plate scale on axis 2 (arcsec per pixel)')
     header['CROTA1'] = (headervals[10], 'WCS field rotation angle on axis 1 (degrees)')
     header['CROTA2'] = (headervals[11], 'WCS field rotation angle on axis 2 (degrees)')
-    
+    header['CCVALD1'] = (headervals[12], 'WCS field center on axis 1 (degrees)')
+    header['CCVALD2'] = (headervals[13], 'WCS field center on axis 2 (degrees)')
+    header['CCVALS1'] = ('{:d}h:{:d}m:{:.2f}s'.format(int(headervals[14]),int(headervals[15]),headervals[16]), 'WCS field center on axis 1 (sexigesimal h m s)')
+    header['CCVALS2'] = ('{:d}d:{:d}am:{:.2f}as'.format(int(headervals[17]),int(headervals[18]),headervals[19]), 'WCS field center on axis 2 (sexigesimal d am as')
+    header['CVAL1RM'] = (headervals[20], 'Mean of WCS residuals on axis 1 (arcsec)')
+    header['CVAL1RS'] = (headervals[21], 'Standard dev of WCS residuals on axis 1 (arcsec')
+    header['CVAL2RM'] = (headervals[22], 'Mean of WCS residuals on axis 2 (arcsec)')
+    header['CVAL2RS'] = (headervals[23], 'Standard dev of WCS residuals on axis 2 (arcsec')
+    header['CVALRM'] = (headervals[24], 'Mean of WCS residuals (arcsec)')
+    header['CVALRS'] = (headervals[25], 'Standard dev of WCS residuals (arcsec)')
+
     outfilename = filename_body + ' WCS.' + filename_ext
     numbered_outfilename = insertCopyNumber(outfilename,filename)
     outfilepath = user_dir+'\\'+numbered_outfilename
@@ -575,6 +592,6 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
 
 def fastrometryCall():
     args = getArgsFromCommandLine()
-    solution = find_WCS(filename=args.filename, ra=args.ra, dec=args.dec, scale=args.scale, scalebnds=args.scalebnds, rotation=args.rotation, rotationbnds=args.rotationbnds, buffer=args.buffer, shape=args.fieldshape, catalogue=args.catalogue, filter=args.filter, pmepoch=args.pmepoch, npts=args.npts, nrefinepts=args.nrefinepts, pixsat=args.pixsat, kernelrad=args.kernelrad, sourcesep=args.sourcesep, vertextol=args.vertextol, nmatchpoints=args.nmatchpoints, nmatchpercent=args.nmatchpercent, save=args.save, load=args.load, verbosity=args.verbosity, debug=args.debug)
+    solution = find_WCS(filename=args.filename, ra=args.ra, dec=args.dec, scale=args.scale, scalebnds=args.scalebnds, rotation=args.rotation, rotationbnds=args.rotationbnds, buffer=args.buffer, shape=args.fieldshape, catalogue=args.catalogue, filter=args.filter, pmepoch=args.pmepoch, npts=args.npts, nrefinepts=args.nrefinepts, pixsat=args.pixsat, kernelrad=args.kernelrad, sourcesep=args.sourcesep, vertextol=args.vertextol, nmatchpoints=args.nmatchpoints, nmatchpercent=args.nmatchpercent, wcsdiagnostics=args.wcsdiagnostics, save=args.save, load=args.load, verbosity=args.verbosity, debug=args.debug)
 
     
\ No newline at end of file
